{% extends "pydata_sphinx_theme/layout.html" %}

{% block fonts %}
<style>
    @import url('https://fonts.googleapis.com/css2?family=Poppins:ital,wght@0,400;0,600;0,700;1,400;1,600;1,700&display=swap');
</style>
{% endblock %}
